"""
* Copyright (C) 2004-2016 Brocade Communications Systems, Inc.  
"""

import sys
from pexpect import pxssh

class SSHSession(pxssh.pxssh):
    """sbsupgrd SSH sessions.
    """

    def __init__(self, config, state, **kwds):
        """This is the constructor.
        """
        super(SSHSession, self).__init__(timeout=3, logfile=sys.stdout, **kwds)
#        With logfile, it is echoed twice, without it is not echoed at all.
#        super(SSHSession, self).__init__(timeout=3, **kwds)
        self.config = config
        self.state = state
        self.NI_SSH_USER_MODE_PROMPT = "SSH@[^>]*[>]"
        self.NI_SSH_PRIV_MODE_PROMPT = "SSH@[^#]*[#]"
        self.PROMPT = self.NI_SSH_USER_MODE_PROMPT

    def connect(self):
        """Connect to the device, and drop into enable mode
        """
        old_logfile = self.logfile
        self.logfile = None

#        print( "u:" + self.config.user)
#        print( "p:" + self.config.passwd)
 
        self.login(self.config.device, self.config.user, self.config.passwd,
            auto_prompt_reset=False)
        self.logfile = old_logfile
 
        # TBD: Check if already enable mode (passwd?), (or) config mode
        self.sendline('')
        if self.prompt():
            self.PROMPT = self.NI_SSH_PRIV_MODE_PROMPT
            self.sendline('enable')
            if not self.prompt():
                raise ExceptionSession("Could not find Prompt:after enable")
        else:
            # Check if enable mode already
            self.PROMPT = self.NI_SSH_PRIV_MODE_PROMPT
            self.sendline('')
            if not self.prompt():
                raise ExceptionSession("Could not find Prompt: unknown")

            
#        self.sendline('skip-page-display')
#        self.prompt()
#        self.sendline('show version')
#        self.prompt()
#        self.PROMPT = self.NI_SSH_USER_MODE_PROMPT
#        self.sendline('exit')
#        self.prompt()

    def disconnect(self):
        """Disconnect connection to the device
           Remember to reset state.session
        """
        self.PROMPT = self.NI_SSH_USER_MODE_PROMPT
        self.sendline('exit')
        self.prompt()
        self.logout()
        
