"""
* Copyright (C) 2004-2016 Brocade Communications Systems, Inc.  
"""

from stage.Stage import *
from argparse import *
import getpass

class ArgP(Stage):
    """ Argument Processor
    """

    def __init__(self, config, state):
        """This is the constructor.
        """
        super(ArgP, self).__init__(config, state)

    def run(self):
        """ Run this stage: Start parsing arguments.
        """

        self.state.logging.info('Stage: Argument Parser: Started')

        parser = ArgumentParser(description="sbsupgrd: Script Upgrade")
        parser.add_argument("-v", "--verbose", action="count", default=0)
        parser.add_argument("--device", required=True,
            help="The device to upgrade")
        parser.add_argument("--manifest", required=True,
            help="The location of the package manifest")
        parser.add_argument("--cf", choices=['slot1', 'slot2'], required=True,
            help="Compact Flash slot1|slot2")
        parser.add_argument("--subdir", required=True,
            help="Destination subdir(eg:61b213)")
        parser.add_argument("--stage", help="Only run this Stage:SanityChecker,ManifestParser,Copier,SignatureValidator,Installer,Cleaner")
        parser.add_argument("--user",
            help="The username to use for login to device")
        parser.add_argument("--passwd",
            help="The password to use for login to device")

        try: 
            args = parser.parse_args()

            self.state.logging.info ("""Arguments:\
 -v {}, --device {}, --manifest {},\
 --cf {}, --subdir {}, --stage {}, --user {}, --passwd ***""".format(
                args.verbose, args.device, args.manifest,
                args.cf, args.subdir, args.stage,
                args.user))

            self.config.verbose = args.verbose
            self.config.device = args.device
            self.config.manifest = args.manifest
            self.config.cf = args.cf
            self.config.stage = args.stage
            self.config.subdir = args.subdir

            if args.user != None:
                self.config.user = args.user
            else:
                self.state.logging.warning(
                 'Stage: Argument Parser: Using logged in username')
                self.config.user = getpass.getuser()

            if args.passwd != None:
                self.state.logging.warning(
                 'Stage: Argument Parser: Insecure. Password on command line.')
                self.config.passwd = args.passwd
            else:
                self.state.logging.info(
                 'Stage: Argument Parser: Prompting for password')
                self.config.passwd = getpass.getpass()
            
        except:
            self.state.logging.error('Stage: Argument Parser: Error')
            exit(2)
 

