"""
* Copyright (C) 2004-2016 Brocade Communications Systems, Inc.  
"""

from stage.Stage import *
from session import *
import sys
import re
import pexpect
import time

class Copier(Stage):
    """ Copier
    """

    def __init__(self, config, state):
        """This is the constructor.
        """
        super(Copier, self).__init__(config, state)

    def run(self):
        """ Run this stage: 1. Make Directory(SESSION), 2. Copy(SCP(SESSION))
        """

        self.state.logging.info('Stage: Copier: Started')

        try: 
            self.make_connection()

            self.make_directories()

            self.copy_files()

            self.break_connection()

        except:
            self.state.logging.error( 'Stage: Copier: Error')
            self.state.count_copy_failed += 1
        finally:
            if self.state.count_copy_failed > 0:
                self.state.logging.error( 'Stage: Copier: SCP Copy Fails')
                self.state.logging.error( self.state.copy_failed)
                self.state.logging.error( 'Stage: Copier: Error')
                exit(4)
 

    def make_directories(self):
        """Make Directories
        """

        self.state.logging.info('Stage: Copier: Make Directories')

        dirs = [ \
            "Boot", \
            "Boot/InterfaceModule", \
            "Boot/ManagementModule", \
            "Combined", \
            "Combined/FPGA", \
            "Combined/Application", \
            "Monitor", \
            "Monitor/InterfaceModule", \
            "Monitor/ManagementModule", \
            "Application", \
            "Application/InterfaceModule", \
            "Application/ManagementModule", \
            "FPGA", \
            "FPGA/InterfaceModule", \
            "FPGA/ManagementModule", \
            "Signatures", \
        ]
 
        subdir = '/' + self.config.cf
        if self.config.subdir != "" :
            subdir = subdir + '/' + self.config.subdir
            self.mkdir(subdir)
        subdir = subdir + self.slash_xmr_mlx
        self.mkdir(subdir)


        for dir in dirs:
            self.mkdir(subdir + '/' + dir)


    def mkdir(self,dir):
        self.state.logging.debug(
                    'Stage: Copier: mkdir ' + dir)
        self.state.session.sendline('mkdir ' + dir)
        self.state.session.prompt()

    def copy_files(self):
        """Copy files
        """

        self.state.logging.info('Stage: Copier: Copying Files')

        r = re.match(r'(.)*/', self.config.manifest)
        src = r.group(0)

        if self.config.subdir != "" :
            subdir = self.config.subdir + self.slash_xmr_mlx
        else:
            subdir = self.xmr_mlx

        for (dir,file) in self.state.filelist:

            # Testing Only
            #if dir != "":
            #   continue;
            # Testing Only

            success = 0
            for tries in range(self.config.max_retry):

                try: 

                    command = 'scp ' \
                        + src[:-1] + dir + '/' + file + ' ' \
                        + self.config.user + '@' + self.config.device \
                        + ':' + self.config.cf + ':' + subdir + dir + '/' + file

                    self.state.logging.debug('Stage: Copier: ' + command)

                    child = pexpect.spawn(command)
                    child.expect('(?i)password', timeout=self.passwd_timeout)
                    child.sendline(self.config.passwd)

                    child.logfile = sys.stdout
                    index = child.expect([pexpect.EOF,
                                          pexpect.TIMEOUT],
                                         timeout=self.scp_timeout)

                    if index == 0:
                        child.isalive()
                        if child.exitstatus == 0:
                            success = 1

                    child.kill(0)
                    if success == 1:
                        break

                except:

                    self.state.logging.info('Stage: Copier: ' + 'Retrying')
                    time.sleep(self.recover_sleep_time)
                    continue

            if success == 0:

                self.state.logging.error(
                    'Stage: Copier: ' + 'Maximum retry exceeded')

                if self.state.copy_failed == None:
                    self.state.copy_failed = ""

                self.state.copy_failed += command 
                self.state.copy_failed += "\n" 

                self.state.count_copy_failed += 1

                if self.state.count_copy_failed >= self.config.max_fails:
                    self.state.logging.error(
                        'Stage: Copier: ' + 'Maximum failures exceeded')
                    raise

