"""
* Copyright (C) 2004-2016 Brocade Communications Systems, Inc.  
"""

from stage.Stage import *
from session import *
import sys
import pexpect

class SanityChecker(Stage):
    """ Sanity Checker
    """

    def __init__(self, config, state):
        """This is the constructor.
        """
        super(SanityChecker, self).__init__(config, state)

    def run(self):
        """ Run this stage: Setup the session, and test.
        """

        self.state.logging.info('Stage: Sanity Checker: Started')
        try: 
            self.make_connection()

            self.get_device_info()

            self.break_connection()

        except:
            self.state.logging.error( 'Stage: Sanity Checker: Error')
            exit(3)
 
    def get_device_info(self):
        """Get the device information
        """

        self.state.logging.info('Stage: Sanity Checker: Get Device Info')

        self.state.session.prompt()

        self.state.session.sendline('skip-page-display')

        self.state.session.prompt()

        self.confirm_xmr_mlx()

        self.confirm_version()

        self.confirm_mr2_mm()

        self.confirm_400m_free()


    def confirm_xmr_mlx(self):
        """Confirm XMR-MLX
        """

        self.state.logging.info('Stage: Sanity Checker: XMR MLX ?')

        self.state.session.sendline('show ver | i processor')

        index = self.state.session.expect_exact(
            ['processor 8544', 'processor 7448',
            pexpect.EOF, pexpect.TIMEOUT])

        if index == 0:
            self.state.logging.info( 'Stage: Sanity Checker: Device: CES/CER')
            self.state.device_type = self.state.CES_CER
            self.state.logging.error( 'No support for CES/CER')
            self.state.logging.error( 'Please use steps in Upgrade Guide')
            self.state.logging.error( 'under \"Performing a basic upgrade\"')
            raise
        elif index == 1:
            self.state.logging.info( 'Stage: Sanity Checker: Device: XMR/MLX')
            self.state.device_type = self.state.XMR_MLX
        elif index == 2: 
            self.state.logging.error(
                'Stage: Sanity Checker: Unknown Device(eof)')
            raise
        elif index == 3:
            self.state.logging.error(
                'Stage: Sanity Checker: Unknown Device(timeout)')
            raise

        self.state.session.prompt()


    def confirm_version(self):
        """Confirm Version (5.7 and above)
        """

        self.state.logging.info('Stage: Sanity Checker: Ver 5.7 and above ?')

        self.state.session.sendline('show ver | i xmr0')

        index = self.state.session.expect_exact(
            ['xmr04',
            'xmr050', 'xmr051', 'xmr052', 'xmr053', 'xmr054', 'xmr055', 'xmr056',
            'xmr057', 'xmr058','xmr059','xmr06','xmr07','xmr08','xmr09','xmr1',
            pexpect.EOF, pexpect.TIMEOUT])

        if index <= 7:
            self.state.logging.info( 'Stage: Sanity Checker: Version: < 5.7')
            self.state.logging.error( 'No support for Version < NI5.7')
            raise
        elif index > 15:
            self.state.logging.info( 'Stage: Sanity Checker: Version: Unknown')
            self.state.logging.error( 'Cannot determine version')
            raise
        else:
            self.state.logging.info(
                'Stage: Sanity Checker: Version Check: Pass')

        self.state.session.prompt()


    def confirm_mr2_mm(self):
        """Confirm MR2 MM
        """

        self.state.logging.info('Stage: Sanity Checker: MR2 MM ?')

        self.state.session.sendline('show ver | i Management Module')

        index = self.state.session.expect_exact(
            ['BR-MLX-MR2',
            pexpect.EOF, pexpect.TIMEOUT])

        if index == 0:
            self.state.logging.info( 'Stage: Sanity Checker: MR2 MM')
        elif index == 1: 
            self.state.logging.error(
                'Stage: Sanity Checker: Unknown Management Module(eof)')
            raise
        elif index == 2:
            self.state.logging.error(
                'Stage: Sanity Checker: Unknown Management Module(timeout)')
            raise

        self.state.session.prompt()


    def confirm_400m_free(self):
        """Confirm 400M free
        """

        self.state.logging.info('Stage: Sanity Checker: 400m free ?')

        if (self.config.stage == "SignatureValidator"):
            self.state.logging.info(
				'Stage: Sanity Checker: Skip. SignatureValidator only')
            return

        if (self.config.stage == "Installer"):
            self.state.logging.info(
				'Stage: Sanity Checker: Skip. Installer only')
            return

        if (self.config.stage == "Cleaner"):
            self.state.logging.info(
				'Stage: Sanity Checker: Skip. Cleaner only')
            return

        self.state.session.sendline('dir /' + self.config.cf)

        index = self.state.session.expect(
            [ r' [,\d]+ bytes free',
            pexpect.EOF, pexpect.TIMEOUT])

        if index == 0:
            line = self.state.session.match.group()
            self.state.logging.info( 'Stage: Sanity Checker: Space '  + line)
            free_space = int(line.replace(',','').replace(' bytes free', ''))
            self.state.logging.info( 'Stage: Sanity Checker: Space Int '  \
                  + str(free_space))

            if free_space < 400000000:
                self.state.logging.error(
                    'Stage: Sanity Checker: Need at least 400,000,000 bytes')
                raise
            else:
                self.state.logging.info( 'Stage: Sanity Checker: Free Space OK')
          
        elif index == 1: 
            self.state.logging.error(
                'Stage: Sanity Checker: Unknown Space available(eof)')
            raise
        elif index == 2:
            self.state.logging.error(
                'Stage: Sanity Checker: Unknown Space available(timeout)')
            raise

        self.state.session.prompt()
