"""
* Copyright (C) 2004-2016 Brocade Communications Systems, Inc.  
"""

from stage.Stage import *
from session import *
import sys
import re
import pexpect
import time

class SignatureValidator(Stage):
    """ Signature Validator
    """

    def __init__(self, config, state):
        """This is the constructor.
        """
        super(SignatureValidator, self).__init__(config, state)

        self.failed_files = []

    def run(self):
        """ Run this stage: 
        """

        try: 
            self.state.logging.info('Stage: SignatureValidator: Started')

            self.make_connection()

            self.check_signatures()

            self.break_connection()
        except:
            self.state.logging.error( 'Stage: SignatureValidator: Error')
            self.failed_files.append("")
        finally:
            if len(self.failed_files):
                self.state.logging.error( 'Stage: SignatureValidator: Fails')
                for i in range(len(self.failed_files)):
                    self.state.logging.error( self.failed_files[i])
                self.state.logging.error( 'Stage: SignatureValidator: Error')
                exit(5)
 
 
    def check_signatures(self):
        """Check Signatures
        """

        self.state.logging.info('Stage: SignatureValidator: Checking')
        self.state.logging.info('Stage: SignatureValidator: *****************')
        self.state.logging.info('Note: These operations may take a few minutes')
        self.state.logging.info('to complete depending on the file size.')
        self.state.logging.info('Stage: SignatureValidator: *****************')

#        r = re.match(r'(.)*/', self.config.manifest)
#        src = r.group(0)

        if self.config.subdir != "" :
            subdir = self.config.subdir + self.slash_xmr_mlx
        else:
            subdir = self.xmr_mlx

        dot_split = re.compile(r'\.')

        for (dir,file) in self.state.filelist:

            if dir == "/Signatures":
                continue

            if dir == "/Combined/Application":
                continue

            file_ext = dot_split.split(file)[1]

            if dir == "" and file_ext != "txt" and file_ext != "mnf":
                continue

            file_with_sha256 = dot_split.split(file)[0] + '.sha256'

            command = 'verify signature file /' + self.config.cf +\
                          '/' + subdir + dir + '/' + file +\
                          ' signature-file /'  + self.config.cf +\
                          '/' + subdir +\
                         ('/Signatures/' if (dir != "") else '/') +\
                          file_with_sha256

            success = 0
            for tries in range(self.config.max_retry):

                try: 

                    self.state.logging.debug('Stage: SignatureValidator: ' + command)

                    self.state.session.prompt()
                    self.state.session.sendline(command)
#                    self.state.session.prompt(self.verify_timeout)

                    index = self.state.session.expect_exact(
                        ['Verified OK', 'Verification Failure',
                         pexpect.EOF, pexpect.TIMEOUT, 'Invalid input'],timeout=self.verify_timeout)

                    success = 1
                    break

                except:

                    self.state.logging.info('Stage: SignatureValidator: ' + 'Retrying')
                    self.break_connection()
                    time.sleep(self.recover_sleep_time)
                    self.make_connection()
                    continue

            if success == 0:

                self.state.logging.error(
                    'Stage: SignatureValidator: ' + 'Maximum retry exceeded')

                index = 5

            if index == 0:
                self.state.logging.debug( 'Stage: SignatureValidator: OK ')
            elif index == 1:
                self.state.logging.debug( 'Stage: SignatureValidator: Failed')
                self.failed_files.append(file)
            elif index == 2: 
                self.state.logging.error(
                    'Stage: SignatureValidator: InvalidResponse(eof)')
                self.failed_files.append(file)
            elif index == 3:
                self.state.logging.error(
                    'Stage: SignatureValidator: InvalidResponse(timeout)')
                self.failed_files.append(file)
            elif index == 4:
                self.state.logging.debug( 'Stage: SignatureValidator: CLI absent ')
            else:
                self.state.logging.debug( 'Stage: SignatureValidator: exception ')
                self.failed_files.append(file)

        if len(self.failed_files):
            raise

