"""
* Copyright (C) 2004-2016 Brocade Communications Systems, Inc.  
"""

from session import *
import time

class Stage(object):
    """All types of sbsupgrd stages use this as a base class.
    """

    def __init__(self, config, state):
        """This is the constructor.
        """
        self.config = config
        self.state = state

        self.slash_xmr_mlx = "/XMR-MLX"
        self.xmr_mlx = "XMR-MLX"
        self.prompt_timeout = 30
        self.prompt_copy_timeout = 3600
        self.passwd_timeout = 30
        self.scp_timeout = 36000
        self.verify_timeout = 300
        self.recover_sleep_time = 2

    def make_connection(self):
        try: 
            if self.state.session is None:
                self.state.logging.info('Stage: Session: inactive')

                success = 0
                for tries in range(self.config.max_retry):
                    try:
                        self.state.session = SSHSession.SSHSession(self.config,\
                                                           self.state)
                        self.state.session.connect()

                        success = 1
                        break
                    except:
                        self.state.session = None
                        self.state.logging.info('Stage: Session: ' + 'Retrying')
                        time.sleep(self.recover_sleep_time)
                        continue

                if success == 0:
                    self.state.logging.error(
                        'Stage: Session: ' + 'Maximum retry exceeded')
                    raise

            else:
                self.state.logging.warning(
                    'Stage: Session: already active')

        except:
            self.state.logging.error( 'Stage: Session: Not Established')
            self.state.logging.error( 'Stage: Session: Check login credentials')
            self.state.logging.error( 'Stage: Session: Error')
            raise
 
    def break_connection(self):
        self.state.logging.info('Stage: Session: disconnecting')
        try:
            self.state.session.disconnect()
        except:
            self.state.logging.info( 'Stage: Session: Exception disconnecting')
        self.state.session = None

